package ACL2::Proceso::ElimDestructores;
use base ACL2::Proceso;

=head1 NAME

ACL2::Proceso::ElimDestructores - proceso de ACL2 de eliminaciE<oacute>n de tE<eacute>rminos destructores

=head1 FUNCTIONS

=cut

#
#    Clase ElimDestructores: proceso de ACL2 de eliminación de términos destructores
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id: ElimDestructores.pm 202 2008-06-27 22:47:53Z bluezio $
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use ACL2::AnalisisLisp;
use ACL2::Orden;
use ACL2::Proceso::Fabrica;

BEGIN {
ACL2::Proceso::Fabrica->registrar
    (
     __PACKAGE__,
     sub {
	 return ACL2::Proceso::ElimDestructores->_identificar(@_);
     }
     );
}

sub escribir_xml_cabecera 
{
    my ($this, $writer) = @_;

    $writer->startTag(ACL2::Tags::ELIMINATE_TAG);

    ACL2::Proceso->volcar_reglas($writer, $this->{rv_reglas});

    # Términos destructores - retiramos los 'and'
    $writer->startTag(ACL2::Tags::DESTRUCTTERMS_TAG);
    foreach (@{$this->{terminos}}) {
	$writer->emptyTag(ACL2::Tags::DESTRUCTTERM_TAG, "name" => $_);
    }
    $writer->endTag(ACL2::Tags::DESTRUCTTERMS_TAG);
    
    # Reemplazos
    $writer->startTag(ACL2::Tags::REPLACEMENTS_TAG);
    while (my ($origen, $destino) = each(%{$this->{reempl}})) {
	$writer->emptyTag(ACL2::Tags::REPLACEMENT_TAG,
			  "from" => $origen, 
			  "to" => $destino);
    }
    $writer->endTag(ACL2::Tags::REPLACEMENTS_TAG);

    ACL2::Orden->salida_parrafos($writer, $this->{salida});
}

sub escribir_xml_final
{
    my ($this, $writer) = @_;
    $writer->endTag(ACL2::Tags::ELIMINATE_TAG);
}


=head2 _identificar

Uso: ME<eacute>todo estE<aacute>tico privado.

Responsabilidad: Realizar la identificaciE<oacute>n de una
SimplificaciE<oacute>n.

ParE<aacute>metros: salida - salida de la meta.

Devuelve: referencia a hash con argumentos para new, o undef si no
coincide.

=cut

sub _identificar {
    caller eq __PACKAGE__ or die;
    my ($paquete, $salida) = @_;
    
    my %res;
    
    $salida =~ 
	/^
	(?-x:\s*The destructor terms? (.*?) can be eliminated by )
	(?-x:using (.*?) to replace ([^\.]+?)\.)

	(?-x: This produces the \w* ?following goals?\.)
	/ox or return undef;

    my ($terminos,$regla,$metodo) = ($1,$2,$3);

    $res{terminos}  = [map { $_ !~ /and|,\s*/i ? $_ : (); } ACL2::AnalisisLisp->dividir_lista("(${terminos})")];
    $res{reempl}    = ACL2::Proceso->obtener_reemplazos($metodo);
    $res{rv_reglas} = [{"type" => ":ELIM", "name" => $regla}];
    $res{salida}    = substr $salida, $-[0], $+[0] - $-[0];
    
    return \%res;
}

1;
